@@ -6,6 +6,7 @@
 #ifndef CONFIG_H_
 #define CONFIG_H_
 #include "tern.h"
+#include "system.h"
 
 tern_node *parse_config_file(char *config_path);
 tern_node *parse_bundled_config(char *config_name);
@@ -17,6 +18,8 @@ void persist_config_at(tern_node *app_config, tern_node *to_save, char *fname);
 void persist_config(tern_node *config);
 char **get_extension_list(tern_node *config, uint32_t *num_exts_out);
 uint32_t get_lowpass_cutoff(tern_node *config);
+tern_node *get_systems_config(void);
+tern_node *get_model(tern_node *config, system_type stype);
 
 #endif //CONFIG_H_
 
