@@ -13,6 +13,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include "vgm.h"
+#include "system.h"
 
 #define MCLKS_NTSC 53693175
 #define MCLKS_PAL  53203395
@@ -24,6 +25,12 @@
 
 #pragma GCC diagnostic ignored "-Wincompatible-pointer-types"
 
+system_header *current_system;
+
+void system_request_exit(system_header *system, uint8_t force_release)
+{
+}
+
 void handle_keydown(int keycode, uint8_t scancode)
 {
 }
